<?php

namespace Ecoyn\Dashboard;

use DateInterval;
use DatePeriod;
use DateTime;
use Exception;
use PDO;
use QUI;
use QUI\Interfaces\System\SystemOutput;
use QUI\System\Output\VoidOutput;

use function floatval;
use function get_class;
use function is_array;
use function json_decode;

class Cron
{
    const CACHE_ORDER_CATEGORIES = 'quiqqer/ecoyn/dashboard/orderCategories/';

    /**
     * @throws Exception
     */
    public static function cron(): void
    {
        self::buildOrderCategoryCache(new VoidOutput());
    }

    /**
     *
     * @param SystemOutput|null $Output
     * @return void
     *
     * @throws Exception
     */
    public static function build(?SystemOutput $Output = null): void
    {
        if ($Output === null) {
            $Output = new VoidOutput();
        }

        self::buildOrderCategoryCache($Output);
    }

    /**
     * @param SystemOutput|null $Output
     * @return void
     *
     * @throws Exception
     */
    public static function buildOrderCategoryCache(?SystemOutput $Output = null): void
    {
        if ($Output === null) {
            $Output = new VoidOutput();
        }

        $PDO = QUI::getDataBase()->getPDO();
        $table = QUI::getDBTableName('orders');

        // day category cache

        $Statement = $PDO->prepare(
            "SELECT id, c_date
             FROM $table
             ORDER BY c_date ASC
             Limit 1"
        );

        $Statement->execute();
        $result = $Statement->fetchAll();

        if (!count($result)) {
            return;
        }

        $firstDate = $result[0]['c_date'];
        $FromDate = new DateTime($firstDate);
        $Today = new DateTime();

        $Interval = new DateInterval('P1D');
        $Period = new DatePeriod($FromDate, $Interval, $Today);

        foreach ($Period as $Day) {
            $day = $Day->format('Y-m-d');
            $dayCache = self::CACHE_ORDER_CATEGORIES . $day;

            try {
                QUI\Cache\LongTermCache::get($dayCache);
                continue;
            } catch (QUI\Exception) {
            }

            self::calcOrderCategoryDayCache($Day, $Output);
        }

        self::calcOrderCategoryDayCache($Today, $Output);
    }

    /**
     * @param $Day
     * @param $Output
     *
     * @return void
     */
    protected static function calcOrderCategoryDayCache($Day, $Output): void
    {
        $PDO = QUI::getDataBase()->getPDO();
        $table = QUI::getDBTableName('orders');

        $day = $Day->format('Y-m-d');
        $dayCache = self::CACHE_ORDER_CATEGORIES . $day;

        $Output->writeLn('Calculate ' . $day);

        $Statement = $PDO->prepare(
            "SELECT *
             FROM $table
             WHERE c_date >= :from AND  c_date <= :to"
        );

        $from = $day . ' 00:00:00';
        $to = $day . ' 23:59:59';

        $Statement->bindParam(':from', $from);
        $Statement->bindParam(':to', $to);
        $Statement->execute();

        $orders = $Statement->fetchAll(PDO::FETCH_ASSOC);
        $result = [];

        foreach ($orders as $order) {
            $articles = json_decode($order['articles'], true);

            if (!is_array($articles) || !$articles) {
                continue;
            }

            if (!isset($articles['articles'])) {
                continue;
            }

            foreach ($articles['articles'] as $article) {
                try {
                    $Product = QUI\ERP\Products\Handler\Products::getProduct((int)$article['id']);
                    $categories = $Product->getCategories();

                    foreach ($categories as $Category) {
                        $catId = $Category->getId();

                        if (get_class($Category) === QUI\ERP\Products\Category\AllProducts::class) {
                            continue;
                        }

                        if (!isset($result[$catId])) {
                            $result[$catId] = [
                                'sum' => 0,
                                'title' => $Category->getTitle()
                            ];
                        }

                        $sum = $result[$catId]['sum'];
                        $sum = $sum + floatval($article['sum']);
                        $result[$catId]['sum'] = $sum;
                    }
                } catch (QUI\Exception) {
                }
            }
        }

        QUI\Cache\LongTermCache::set($dayCache, $result);
    }
}
